@@ -2,8 +2,8 @@
  * Audacious CD Digital Audio plugin
  *
  * Copyright (c) 2007 Calin Crisan <ccrisan@gmail.com>
- * Copyright 2009 John Lindgren
- * Copyright 2009 Tomasz Moń <desowin@gmail.com>
+ * Copyright (c) 2009-2012 John Lindgren <john.lindgren@aol.com>
+ * Copyright (c) 2009 Tomasz Moń <desowin@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
+#include <pthread.h>
 #include <string.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <errno.h>
 
 #include <cdio/cdio.h>
@@ -49,13 +49,17 @@
 #include <audacious/misc.h>
 #include <audacious/playlist.h>
 #include <audacious/plugin.h>
+#include <audacious/preferences.h>
 #include <libaudcore/hook.h>
 #include <libaudgui/libaudgui.h>
 #include <libaudgui/libaudgui-gtk.h>
 
-#include "cdaudio-ng.h"
 #include "config.h"
-#include "configure.h"
+
+#define DEF_STRING_LEN 256
+
+#define MIN_DISC_SPEED 2
+#define MAX_DISC_SPEED 24
 
 #define MAX_RETRIES 10
 #define MAX_SKIPS 10
@@ -64,68 +68,106 @@
 
 typedef struct
 {
-    gchar performer[DEF_STRING_LEN];
-    gchar name[DEF_STRING_LEN];
-    gchar genre[DEF_STRING_LEN];
-    gint startlsn;
-    gint endlsn;
+    char performer[DEF_STRING_LEN];
+    char name[DEF_STRING_LEN];
+    char genre[DEF_STRING_LEN];
+    int startlsn;
+    int endlsn;
 }
 trackinfo_t;
 
-static GMutex *mutex;
-static gint seek_time;
-static gboolean stop_flag;
+static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+static int seek_time;
+static bool_t playing;
 
 /* lock mutex to read / set these variables */
-cdng_cfg_t cdng_cfg;
-static gint firsttrackno = -1;
-static gint lasttrackno = -1;
-static gint n_audio_tracks;
+static int firsttrackno = -1;
+static int lasttrackno = -1;
+static int n_audio_tracks;
 static cdrom_drive_t *pcdrom_drive = NULL;
 static trackinfo_t *trackinfo = NULL;
-static gint monitor_source = 0;
+static int monitor_source = 0;
 
-/* read / set these variables in main thread only */
-
-static gboolean cdaudio_init (void);
-static void cdaudio_about (void);
-static void cdaudio_configure (void);
-static gint cdaudio_is_our_file (const gchar * filename, VFSFile * file);
-static gboolean cdaudio_play (InputPlayback * p, const gchar * name, VFSFile *
- file, gint start, gint stop, gboolean pause);
+static bool_t cdaudio_init (void);
+static int cdaudio_is_our_file (const char * filename, VFSFile * file);
+static bool_t cdaudio_play (InputPlayback * p, const char * name, VFSFile *
+ file, int start, int stop, bool_t pause);
 static void cdaudio_stop (InputPlayback * pinputplayback);
-static void cdaudio_pause (InputPlayback * p, gboolean paused);
-static void cdaudio_mseek (InputPlayback * p, gint time);
+static void cdaudio_pause (InputPlayback * p, bool_t paused);
+static void cdaudio_mseek (InputPlayback * p, int time);
 static void cdaudio_cleanup (void);
-static Tuple * make_tuple (const gchar * filename, VFSFile * file);
+static Tuple * make_tuple (const char * filename, VFSFile * file);
 static void scan_cd (void);
-static void refresh_trackinfo (gboolean warning);
-static gint calculate_track_length (gint startlsn, gint endlsn);
-static gint find_trackno_from_filename (const gchar * filename);
+static void refresh_trackinfo (bool_t warning);
+static int calculate_track_length (int startlsn, int endlsn);
+static int find_trackno_from_filename (const char * filename);
 
-static const gchar * const schemes[] = {"cdda", NULL};
+static const char cdaudio_about[] =
+ N_("Copyright (C) 2007-2012 Calin Crisan <ccrisan@gmail.com> and others.\n\n"
+    "Many thanks to libcdio developers <http://www.gnu.org/software/libcdio/>\n"
+    "and to libcddb developers <http://libcddb.sourceforge.net/>.\n\n"
+    "Also thank you to Tony Vroon for mentoring and guiding me.\n\n"
+    "This was a Google Summer of Code 2007 project.");
+
+static const char * const schemes[] = {"cdda", NULL};
+
+static const char * const cdaudio_defaults[] = {
+ "disc_speed", "2",
+ "use_cdtext", "TRUE",
+ "use_cddb", "TRUE",
+ "cddbhttp", "FALSE",
+ "cddbserver", "freedb.org",
+ "cddbport", "8880",
+ NULL};
+
+static const PreferencesWidget cdaudio_widgets[] = {
+ {WIDGET_LABEL, N_("<b>Device</b>")},
+ {WIDGET_SPIN_BTN, N_("Read speed:"),
+  .cfg_type = VALUE_INT, .csect = "CDDA", .cname = "disc_speed",
+  .data = {.spin_btn = {MIN_DISC_SPEED, MAX_DISC_SPEED, 1}}},
+ {WIDGET_ENTRY, N_("Override device:"),
+  .cfg_type = VALUE_STRING, .csect = "CDDA", .cname = "device"},
+ {WIDGET_LABEL, N_("<b>Metadata</b>")},
+ {WIDGET_CHK_BTN, N_("Use CD-Text"),
+  .cfg_type = VALUE_BOOLEAN, .csect = "CDDA", .cname = "use_cdtext"},
+ {WIDGET_CHK_BTN, N_("Use CDDB"),
+  .cfg_type = VALUE_BOOLEAN, .csect = "CDDA", .cname = "use_cddb"},
+ {WIDGET_CHK_BTN, N_("Use HTTP instead of CDDBP"), .child = TRUE,
+  .cfg_type = VALUE_BOOLEAN, .csect = "CDDA", .cname = "cddbhttp"},
+ {WIDGET_ENTRY, N_("Server:"), .child = TRUE,
+  .cfg_type = VALUE_STRING, .csect = "CDDA", .cname = "cddbserver"},
+ {WIDGET_ENTRY, N_("Path:"), .child = TRUE,
+  .cfg_type = VALUE_STRING, .csect = "CDDA", .cname = "cddbpath"},
+ {WIDGET_SPIN_BTN, N_("Port:"), .child = TRUE,
+  .cfg_type = VALUE_INT, .csect = "CDDA", .cname = "cddbport",
+  .data = {.spin_btn = {0, 65535, 1}}}};
+
+static const PluginPreferences cdaudio_prefs = {
+ .widgets = cdaudio_widgets,
+ .n_widgets = G_N_ELEMENTS (cdaudio_widgets)};
 
 AUD_INPUT_PLUGIN
 (
-    .name = "Audio CD Support",
+    .name = N_("Audio CD Plugin"),
+    .domain = PACKAGE,
+    .about_text = cdaudio_about,
+    .prefs = & cdaudio_prefs,
     .init = cdaudio_init,
-    .about = cdaudio_about,
-    .configure = cdaudio_configure,
+    .cleanup = cdaudio_cleanup,
     .is_our_file_from_vfs = cdaudio_is_our_file,
     .play = cdaudio_play,
     .stop = cdaudio_stop,
     .pause = cdaudio_pause,
     .mseek = cdaudio_mseek,
-    .cleanup = cdaudio_cleanup,
     .probe_for_tuple = make_tuple,
     .schemes = schemes,
     .have_subtune = TRUE,
 )
 
-static void cdaudio_error (const gchar * message_format, ...)
+static void cdaudio_error (const char * message_format, ...)
 {
     va_list args;
-    gchar *msg = NULL;
+    char *msg = NULL;
 
     va_start (args, message_format);
     msg = g_markup_vprintf_escaped (message_format, args);
@@ -136,13 +178,13 @@ static void cdaudio_error (const gchar * message_format, ...)
 }
 
 /* main thread only */
-static void purge_playlist (gint playlist)
+static void purge_playlist (int playlist)
 {
-    gint length = aud_playlist_entry_count (playlist);
+    int length = aud_playlist_entry_count (playlist);
 
-    for (gint count = 0; count < length; count ++)
+    for (int count = 0; count < length; count ++)
     {
-        gchar * filename = aud_playlist_entry_get_filename (playlist, count);
+        char * filename = aud_playlist_entry_get_filename (playlist, count);
 
         if (cdaudio_is_our_file (filename, NULL))
         {
@@ -158,29 +200,36 @@ static void purge_playlist (gint playlist)
 /* main thread only */
 static void purge_all_playlists (void)
 {
-    gint playlists = aud_playlist_count ();
-    gint count;
+    int playlists = aud_playlist_count ();
+    int count;
 
     for (count = 0; count < playlists; count++)
         purge_playlist (count);
 }
 
 /* main thread only */
-static gboolean monitor (gpointer unused)
+static bool_t monitor (gpointer unused)
 {
-    g_mutex_lock (mutex);
+    pthread_mutex_lock (& mutex);
+
+    /* make sure not to close drive handle while playing */
+    if (playing)
+    {
+        pthread_mutex_unlock (& mutex);
+        return true;
+    }
 
     if (trackinfo != NULL)
         refresh_trackinfo (FALSE);
 
     if (trackinfo != NULL)
     {
-        g_mutex_unlock (mutex);
+        pthread_mutex_unlock (& mutex);
         return TRUE;
     }
 
     monitor_source = 0;
-    g_mutex_unlock (mutex);
+    pthread_mutex_unlock (& mutex);
 
     purge_all_playlists ();
     return FALSE;
@@ -189,51 +238,18 @@ static gboolean monitor (gpointer unused)
 /* mutex must be locked */
 static void trigger_monitor (void)
 {
-    if (monitor_source)
-        return;
-
-#if GLIB_CHECK_VERSION (2, 14, 0)
-    monitor_source = g_timeout_add_seconds (1, monitor, NULL);
-#else
-    monitor_source = g_timeout_add (1000, monitor, NULL);
-#endif
+    if (! monitor_source)
+        monitor_source = g_timeout_add_seconds (1, monitor, NULL);
 }
 
-static const gchar * const cdaudio_defaults[] = {
- "use_cdtext", "TRUE",
- "use_cddb", "TRUE",
- "cddbserver", "freedb.org",
- "cddbport", "8880",
- "cddbhttp", "FALSE",
- "disc_speed", "2",
- NULL};
-
 /* main thread only */
-static gboolean cdaudio_init (void)
+static bool_t cdaudio_init (void)
 {
-    mutex = g_mutex_new ();
-
     aud_config_set_defaults ("CDDA", cdaudio_defaults);
 
-    cdng_cfg.use_cdtext = aud_get_bool ("CDDA", "use_cdtext");
-    cdng_cfg.use_cddb = aud_get_bool ("CDDA", "use_cddb");
-    cdng_cfg.device = aud_get_string ("CDDA", "device");
-    cdng_cfg.cddb_server = aud_get_string ("CDDA", "cddbserver");
-    cdng_cfg.cddb_path = aud_get_string ("CDDA", "cddbpath");
-    cdng_cfg.cddb_port = aud_get_int ("CDDA", "cddbport");
-    cdng_cfg.cddb_http = aud_get_bool ("CDDA", "cddbhttp");
-    cdng_cfg.disc_speed = aud_get_int ("CDDA", "disc_speed");
-    cdng_cfg.disc_speed = CLAMP (cdng_cfg.disc_speed, MIN_DISC_SPEED, MAX_DISC_SPEED);
-
-    cdng_cfg.use_proxy = aud_get_bool (NULL, "use_proxy");
-    cdng_cfg.proxy_host = aud_get_string (NULL, "proxy_host");
-    cdng_cfg.proxy_port = aud_get_int (NULL, "proxy_port");
-    cdng_cfg.proxy_username = aud_get_string (NULL, "proxy_user");
-    cdng_cfg.proxy_password = aud_get_string (NULL, "proxy_pass");
-
     if (!cdio_init ())
     {
-        cdaudio_error ("Failed to initialize cdio subsystem.");
+        cdaudio_error (_("Failed to initialize cdio subsystem."));
         return FALSE;
     }
 
@@ -242,37 +258,16 @@ static gboolean cdaudio_init (void)
     return TRUE;
 }
 
-/* main thread only */
-static void cdaudio_about (void)
-{
-    static GtkWidget * about_window = NULL;
-
-    audgui_simple_message (& about_window, GTK_MESSAGE_INFO,
-     _("About Audio CD Plugin"),
-     _("Copyright (c) 2007, by Calin Crisan <ccrisan@gmail.com> and The Audacious Team.\n\n"
-     "Many thanks to libcdio developers <http://www.gnu.org/software/libcdio/>\n"
-     "\tand to libcddb developers <http://libcddb.sourceforge.net/>.\n\n"
-     "Also thank you Tony Vroon for mentoring & guiding me.\n\n"
-     "This was a Google Summer of Code 2007 project.\n\n"
-     "Copyright 2009 John Lindgren"));
-}
-
-/* main thread only */
-static void cdaudio_configure ()
-{
-    configure_show_gui ();
-}
-
 /* thread safe (mutex may be locked) */
-static gint cdaudio_is_our_file (const gchar * filename, VFSFile * file)
+static int cdaudio_is_our_file (const char * filename, VFSFile * file)
 {
     return !strncmp (filename, "cdda://", 7);
 }
 
 /* thread safe (mutex may be locked) */
 static void cdaudio_set_strinfo (trackinfo_t * t,
-                                 const gchar * performer, const gchar * name,
-                                 const gchar * genre)
+                                 const char * performer, const char * name,
+                                 const char * genre)
 {
     g_strlcpy (t->performer, performer, DEF_STRING_LEN);
     g_strlcpy (t->name, name, DEF_STRING_LEN);
@@ -282,8 +277,8 @@ static void cdaudio_set_strinfo (trackinfo_t * t,
 /* thread safe (mutex may be locked) */
 static void cdaudio_set_fullinfo (trackinfo_t * t,
                                   const lsn_t startlsn, const lsn_t endlsn,
-                                  const gchar * performer, const gchar * name,
-                                  const gchar * genre)
+                                  const char * performer, const char * name,
+                                  const char * genre)
 {
     t->startlsn = startlsn;
     t->endlsn = endlsn;
@@ -291,10 +286,10 @@ static void cdaudio_set_fullinfo (trackinfo_t * t,
 }
 
 /* play thread only */
-static gboolean cdaudio_play (InputPlayback * p, const gchar * name, VFSFile *
- file, gint start, gint stop, gboolean pause)
+static bool_t cdaudio_play (InputPlayback * p, const char * name, VFSFile *
+ file, int start, int stop, bool_t pause)
 {
-    g_mutex_lock (mutex);
+    pthread_mutex_lock (& mutex);
 
     if (trackinfo == NULL)
     {
@@ -302,44 +297,36 @@ static gboolean cdaudio_play (InputPlayback * p, const gchar * name, VFSFile *
 
         if (trackinfo == NULL)
         {
-ERR:
-            g_mutex_unlock (mutex);
+            pthread_mutex_unlock (& mutex);
             return FALSE;
         }
     }
 
-    gint trackno = find_trackno_from_filename (name);
+    bool_t okay = FALSE;
+    int trackno = find_trackno_from_filename (name);
 
     if (trackno < 0)
+        cdaudio_error (_("Invalid URI %s."), name);
+    else if (trackno < firsttrackno || trackno > lasttrackno)
+        cdaudio_error (_("Track %d not found."), trackno);
+    else if (! cdda_track_audiop (pcdrom_drive, trackno))
+        cdaudio_error (_("Track %d is a data track."), trackno);
+    else if (! p->output->open_audio (FMT_S16_LE, 44100, 2))
+        cdaudio_error (_("Failed to open audio output."));
+    else
+        okay = TRUE;
+
+    if (! okay)
     {
-        cdaudio_error ("Invalid URI %s.", name);
-        goto ERR;
+        pthread_mutex_unlock (& mutex);
+        return FALSE;
     }
 
-    if (trackno < firsttrackno || trackno > lasttrackno)
-    {
-        cdaudio_error ("Track %d not found.", trackno);
-        goto ERR;
-    }
-
-    /* don't play any data tracks */
-    if (!cdda_track_audiop (pcdrom_drive, trackno))
-    {
-        cdaudio_error ("Track %d is a data track.\n", trackno);
-        goto ERR;
-    }
-
-    gint startlsn = trackinfo[trackno].startlsn;
-    gint endlsn = trackinfo[trackno].endlsn;
-
-    if (! p->output->open_audio (FMT_S16_LE, 44100, 2))
-    {
-        cdaudio_error ("Failed to open audio output.");
-        goto ERR;
-    }
+    int startlsn = trackinfo[trackno].startlsn;
+    int endlsn = trackinfo[trackno].endlsn;
 
     seek_time = (start > 0) ? start : -1;
-    stop_flag = FALSE;
+    playing = TRUE;
 
     if (stop >= 0)
         endlsn = MIN (endlsn, startlsn + stop * 75 / 1000);
@@ -350,24 +337,16 @@ ERR:
     p->set_params (p, 1411200, 44100, 2);
     p->set_pb_ready (p);
 
-    g_mutex_unlock (mutex);
+    int buffer_size = aud_get_int (NULL, "output_buffer_size");
+    int speed = aud_get_int ("CDDA", "disc_speed");
+    speed = CLAMP (speed, MIN_DISC_SPEED, MAX_DISC_SPEED);
+    int sectors = CLAMP (buffer_size / 2, 50, 250) * speed * 75 / 1000;
+    unsigned char buffer[2352 * sectors];
+    int currlsn = startlsn;
+    int retry_count = 0, skip_count = 0;
 
-    gint buffer_size = aud_get_int (NULL, "output_buffer_size");
-    gint sectors = CLAMP (buffer_size / 2, 50, 250) * cdng_cfg.disc_speed * 75 / 1000;
-    guchar buffer[2352 * sectors];
-    gint currlsn = startlsn;
-    gint retry_count = 0, skip_count = 0;
-
-    while (1)
+    while (playing)
     {
-        g_mutex_lock (mutex);
-
-        if (stop_flag)
-        {
-            g_mutex_unlock (mutex);
-            goto CLOSE;
-        }
-
         if (seek_time >= 0)
         {
             p->output->flush (seek_time);
@@ -375,102 +354,88 @@ ERR:
             seek_time = -1;
         }
 
-        g_mutex_unlock (mutex);
-
         sectors = MIN (sectors, endlsn + 1 - currlsn);
         if (sectors < 1)
             break;
 
-        if (cdio_read_audio_sectors (pcdrom_drive->p_cdio, buffer, currlsn, sectors) ==
-         DRIVER_OP_SUCCESS)
+        /* unlock mutex here to avoid blocking
+         * other threads must be careful not to close drive handle */
+        pthread_mutex_unlock (& mutex);
+
+        int ret = cdio_read_audio_sectors (pcdrom_drive->p_cdio, buffer,
+         currlsn, sectors);
+
+        if (ret == DRIVER_OP_SUCCESS)
+            p->output->write_audio (buffer, 2352 * sectors);
+
+        pthread_mutex_lock (& mutex);
+
+        if (ret == DRIVER_OP_SUCCESS)
         {
+            currlsn += sectors;
             retry_count = 0;
             skip_count = 0;
         }
         else if (sectors > 16)
         {
-            warn ("Read failed; reducing read size.\n");
+            /* maybe a smaller read size will help */
             sectors /= 2;
-            continue;
         }
         else if (retry_count < MAX_RETRIES)
         {
-            warn ("Read failed; retrying.\n");
+            /* still failed; retry a few times */
             retry_count ++;
-            continue;
         }
         else if (skip_count < MAX_SKIPS)
         {
-            warn ("Read failed; skipping.\n");
+            /* maybe the disk is scratched; try skipping ahead */
             currlsn = MIN (currlsn + 75, endlsn + 1);
             skip_count ++;
-            continue;
         }
         else
         {
-            cdaudio_error ("Too many read errors; giving up.");
+            /* still failed; give it up */
+            cdaudio_error (_("Error reading audio CD."));
             break;
         }
-
-        p->output->write_audio (buffer, 2352 * sectors);
-        currlsn += sectors;
     }
 
-    while (p->output->buffer_playing ())
-        g_usleep (20000);
+    playing = FALSE;
 
-    g_mutex_lock (mutex);
-    stop_flag = FALSE;
-    g_mutex_unlock (mutex);
-
-CLOSE:
-    p->output->close_audio ();
+    pthread_mutex_unlock (& mutex);
     return TRUE;
 }
 
 /* main thread only */
 static void cdaudio_stop (InputPlayback * p)
 {
-    g_mutex_lock (mutex);
-
-    if (! stop_flag)
-    {
-        stop_flag = TRUE;
-        p->output->abort_write();
-    }
-
-    g_mutex_unlock (mutex);
+    pthread_mutex_lock (& mutex);
+    playing = FALSE;
+    p->output->abort_write();
+    pthread_mutex_unlock (& mutex);
 }
 
 /* main thread only */
-static void cdaudio_pause (InputPlayback * p, gboolean pause)
+static void cdaudio_pause (InputPlayback * p, bool_t pause)
 {
-    g_mutex_lock (mutex);
-
-    if (! stop_flag)
-        p->output->pause (pause);
-
-    g_mutex_unlock (mutex);
+    pthread_mutex_lock (& mutex);
+    p->output->pause (pause);
+    pthread_mutex_unlock (& mutex);
 }
 
 /* main thread only */
-static void cdaudio_mseek (InputPlayback * p, gint time)
+static void cdaudio_mseek (InputPlayback * p, int time)
 {
-    g_mutex_lock (mutex);
-
-    if (! stop_flag)
-    {
-        seek_time = time;
-        p->output->abort_write();
-    }
-
-    g_mutex_unlock (mutex);
+    pthread_mutex_lock (& mutex);
+    seek_time = time;
+    p->output->abort_write();
+    pthread_mutex_unlock (& mutex);
 }
 
 /* main thread only */
 static void cdaudio_cleanup (void)
 {
-    g_mutex_lock (mutex);
+    pthread_mutex_lock (& mutex);
 
     if (monitor_source)
     {
@@ -483,6 +448,7 @@ static void cdaudio_cleanup (void)
         cdda_close (pcdrom_drive);
         pcdrom_drive = NULL;
     }
+
     if (trackinfo != NULL)
     {
         g_free (trackinfo);
@@ -491,35 +457,16 @@ static void cdaudio_cleanup (void)
 
     libcddb_shutdown ();
 
-    // todo: destroy the gui
-
-    aud_set_bool ("CDDA", "use_cdtext", cdng_cfg.use_cdtext);
-    aud_set_bool ("CDDA", "use_cddb", cdng_cfg.use_cddb);
-    aud_set_string ("CDDA", "device", cdng_cfg.device);
-    aud_set_string ("CDDA", "cddbserver", cdng_cfg.cddb_server);
-    aud_set_string ("CDDA", "cddbpath", cdng_cfg.cddb_path);
-    aud_set_int ("CDDA", "cddbport", cdng_cfg.cddb_port);
-    aud_set_bool ("CDDA", "cddbhttp", cdng_cfg.cddb_http);
-    aud_set_int ("CDDA", "disc_speed", cdng_cfg.disc_speed);
-
-    g_free (cdng_cfg.device);
-    g_free (cdng_cfg.cddb_server);
-    g_free (cdng_cfg.cddb_path);
-    g_free (cdng_cfg.proxy_host);
-    g_free (cdng_cfg.proxy_username);
-    g_free (cdng_cfg.proxy_password);
-
-    g_mutex_unlock (mutex);
-    g_mutex_free (mutex);
+    pthread_mutex_unlock (& mutex);
 }
 
 /* thread safe */
-static Tuple * make_tuple (const gchar * filename, VFSFile * file)
+static Tuple * make_tuple (const char * filename, VFSFile * file)
 {
     Tuple *tuple = NULL;
-    gint trackno;
+    int trackno;
 
-    g_mutex_lock (mutex);
+    pthread_mutex_lock (& mutex);
 
     if (trackinfo == NULL)
         refresh_trackinfo (TRUE);
@@ -530,8 +477,8 @@ static Tuple * make_tuple (const gchar * filename, VFSFile * file)
     {
         tuple = tuple_new_from_filename (filename);
 
-        gint subtunes[n_audio_tracks];
-        gint i = 0;
+        int subtunes[n_audio_tracks];
+        int i = 0;
 
         /* only add the audio tracks to the playlist */
         for (trackno = firsttrackno; trackno <= lasttrackno; trackno++)
@@ -559,39 +506,21 @@ static Tuple * make_tuple (const gchar * filename, VFSFile * file)
 
     tuple = tuple_new_from_filename (filename);
     tuple_set_format (tuple, _("Audio CD"), 2, 44100, 1411);
-
-    if (strlen (trackinfo[trackno].performer))
-    {
-        tuple_set_str (tuple, FIELD_ARTIST, NULL,
-                                    trackinfo[trackno].performer);
-    }
-    if (strlen (trackinfo[0].name))
-    {
-        tuple_set_str (tuple, FIELD_ALBUM, NULL,
-                                    trackinfo[0].name);
-    }
-    if (strlen (trackinfo[trackno].name))
-    {
-        tuple_set_str (tuple, FIELD_TITLE, NULL,
-                                    trackinfo[trackno].name);
-    }
-
     tuple_set_int (tuple, FIELD_TRACK_NUMBER, NULL, trackno);
+    tuple_set_int (tuple, FIELD_LENGTH, NULL, calculate_track_length
+     (trackinfo[trackno].startlsn, trackinfo[trackno].endlsn));
 
-    tuple_set_int (tuple, FIELD_LENGTH, NULL,
-                             calculate_track_length (trackinfo[trackno].
-                                                     startlsn,
-                                                     trackinfo[trackno].
-                                                     endlsn));
-
-    if (strlen (trackinfo[trackno].genre))
-    {
-        tuple_set_str (tuple, FIELD_GENRE, NULL,
-                                    trackinfo[trackno].genre);
-    }
+    if (trackinfo[trackno].performer[0])
+        tuple_set_str (tuple, FIELD_ARTIST, NULL, trackinfo[trackno].performer);
+    if (trackinfo[0].name[0])
+        tuple_set_str (tuple, FIELD_ALBUM, NULL, trackinfo[0].name);
+    if (trackinfo[trackno].name[0])
+        tuple_set_str (tuple, FIELD_TITLE, NULL, trackinfo[trackno].name);
+    if (trackinfo[trackno].genre[0])
+        tuple_set_str (tuple, FIELD_GENRE, NULL, trackinfo[trackno].genre);
 
   DONE:
-    g_mutex_unlock (mutex);
+    pthread_mutex_unlock (& mutex);
     return tuple;
 }
 
@@ -601,41 +530,30 @@ static void open_cd (void)
     AUDDBG ("Opening CD drive.\n");
     g_return_if_fail (pcdrom_drive == NULL);
 
-    /* find an available, audio capable, cd drive  */
-    if (cdng_cfg.device != NULL && strlen (cdng_cfg.device) > 0)
+    char * device = aud_get_string ("CDDA", "device");
+
+    if (device[0])
     {
-        pcdrom_drive = cdda_identify (cdng_cfg.device, 1, NULL);
-        if (pcdrom_drive == NULL)
-        {
-            cdaudio_error ("Failed to open CD device \"%s\".", cdng_cfg.device);
-            return;
-        }
+        if (! (pcdrom_drive = cdda_identify (device, 1, NULL)))
+            cdaudio_error (_("Failed to open CD device %s."), device);
     }
     else
     {
-        gchar **ppcd_drives =
-            cdio_get_devices_with_cap (NULL, CDIO_FS_AUDIO, false);
+        char * * ppcd_drives = cdio_get_devices_with_cap (NULL, CDIO_FS_AUDIO, FALSE);
 
-        if (ppcd_drives != NULL && *ppcd_drives != NULL)
-        {                       /* we have at least one audio capable cd drive */
-            pcdrom_drive = cdda_identify (*ppcd_drives, 1, NULL);
-            if (pcdrom_drive == NULL)
-            {
-                cdaudio_error ("Failed to open CD.");
-                return;
-            }
-            AUDDBG ("found cd drive \"%s\" with audio capable media\n",
-                   *ppcd_drives);
+        if (ppcd_drives && ppcd_drives[0])
+        {
+            if (! (pcdrom_drive = cdda_identify (ppcd_drives[0], 1, NULL)))
+                cdaudio_error (_("Failed to open CD device %s."), ppcd_drives[0]);
         }
         else
-        {
-            cdaudio_error ("No audio capable CD drive found.");
-            return;
-        }
+            cdaudio_error (_("No audio capable CD drive found."));
 
-        if (ppcd_drives != NULL && *ppcd_drives != NULL)
+        if (ppcd_drives)
             cdio_free_device_list (ppcd_drives);
     }
+
+    free (device);
 }
 
 /* mutex must be locked */
@@ -645,7 +563,7 @@ static void scan_cd (void)
     g_return_if_fail (pcdrom_drive != NULL);
     g_return_if_fail (trackinfo == NULL);
 
-    gint trackno;
+    int trackno;
 
     /* general track initialization */
 
@@ -656,18 +574,20 @@ static void scan_cd (void)
     /* finish initialization of drive/disc (performs disc TOC sanitization) */
     if (cdda_open (pcdrom_drive) != 0)
     {
-        cdaudio_error ("Failed to finish initializing opened CD drive.");
+        cdaudio_error (_("Failed to finish initializing opened CD drive."));
         goto ERR;
     }
 
-    if (cdda_speed_set (pcdrom_drive, cdng_cfg.disc_speed) != DRIVER_OP_SUCCESS)
+    int speed = aud_get_int ("CDDA", "disc_speed");
+    speed = CLAMP (speed, MIN_DISC_SPEED, MAX_DISC_SPEED);
+    if (cdda_speed_set (pcdrom_drive, speed) != DRIVER_OP_SUCCESS)
         warn ("Cannot set drive speed.\n");
 
     firsttrackno = cdio_get_first_track_num (pcdrom_drive->p_cdio);
     lasttrackno = cdio_get_last_track_num (pcdrom_drive->p_cdio);
     if (firsttrackno == CDIO_INVALID_TRACK || lasttrackno == CDIO_INVALID_TRACK)
     {
-        cdaudio_error ("Failed to retrieve first/last track number.");
+        cdaudio_error (_("Failed to retrieve first/last track number."));
         goto ERR;
     }
     AUDDBG ("first track is %d and last track is %d\n", firsttrackno,
@@ -692,7 +612,7 @@ static void scan_cd (void)
         if (trackinfo[trackno].startlsn == CDIO_INVALID_LSN
             || trackinfo[trackno].endlsn == CDIO_INVALID_LSN)
         {
-            cdaudio_error ("Cannot read start/end LSN for track %d.", trackno);
+            cdaudio_error (_("Cannot read start/end LSN for track %d."), trackno);
             goto ERR;
         }
 
@@ -702,7 +622,7 @@ static void scan_cd (void)
     }
 
     /* get trackinfo[0] cdtext information (the disc) */
-    if (cdng_cfg.use_cdtext)
+    if (aud_get_bool ("CDDA", "use_cdtext"))
     {
         AUDDBG ("getting cd-text information for disc\n");
         cdtext_t *pcdtext = cdio_get_cdtext (pcdrom_drive->p_cdio, 0);
@@ -723,11 +643,11 @@ static void scan_cd (void)
     }
 
     /* get track information from cdtext */
-    gboolean cdtext_was_available = FALSE;
+    bool_t cdtext_was_available = FALSE;
     for (trackno = firsttrackno; trackno <= lasttrackno; trackno++)
     {
         cdtext_t *pcdtext = NULL;
-        if (cdng_cfg.use_cdtext)
+        if (aud_get_bool ("CDDA", "use_cdtext"))
         {
             AUDDBG ("getting cd-text information for track %d\n", trackno);
             pcdtext = cdio_get_cdtext (pcdrom_drive->p_cdio, trackno);
@@ -765,11 +685,11 @@ static void scan_cd (void)
         cddb_track_t *pcddb_track = NULL;
         lba_t lba;              /* Logical Block Address */
 
-        if (cdng_cfg.use_cddb)
+        if (aud_get_bool ("CDDA", "use_cddb"))
         {
             pcddb_conn = cddb_new ();
             if (pcddb_conn == NULL)
-                cdaudio_error ("Failed to create the cddb connection.");
+                cdaudio_error (_("Failed to create the cddb connection."));
             else
             {
                 AUDDBG ("getting CDDB info\n");
@@ -777,33 +697,46 @@ static void scan_cd (void)
                 cddb_cache_enable (pcddb_conn);
                 // cddb_cache_set_dir(pcddb_conn, "~/.cddbslave");
 
-                if (cdng_cfg.use_proxy)
+                char * server = aud_get_string ("CDDA", "cddbserver");
+                char * path = aud_get_string ("CDDA", "cddbpath");
+                int port = aud_get_int ("CDDA", "cddbport");
+
+                if (aud_get_bool (NULL, "use_proxy"))
                 {
+                    char * prhost = aud_get_string (NULL, "proxy_host");
+                    int prport = aud_get_int (NULL, "proxy_port");
+                    char * pruser = aud_get_string (NULL, "proxy_user");
+                    char * prpass = aud_get_string (NULL, "proxy_pass");
+
                     cddb_http_proxy_enable (pcddb_conn);
-                    cddb_set_http_proxy_server_name (pcddb_conn,
-                                                     cdng_cfg.proxy_host);
-                    cddb_set_http_proxy_server_port (pcddb_conn,
-                                                     cdng_cfg.proxy_port);
-                    cddb_set_http_proxy_username (pcddb_conn,
-                                                  cdng_cfg.proxy_username);
-                    cddb_set_http_proxy_password (pcddb_conn,
-                                                  cdng_cfg.proxy_password);
-                    cddb_set_server_name (pcddb_conn, cdng_cfg.cddb_server);
-                    cddb_set_server_port (pcddb_conn, cdng_cfg.cddb_port);
+                    cddb_set_http_proxy_server_name (pcddb_conn, prhost);
+                    cddb_set_http_proxy_server_port (pcddb_conn, prport);
+                    cddb_set_http_proxy_username (pcddb_conn, pruser);
+                    cddb_set_http_proxy_password (pcddb_conn, prpass);
+
+                    free (prhost);
+                    free (pruser);
+                    free (prpass);
+
+                    cddb_set_server_name (pcddb_conn, server);
+                    cddb_set_server_port (pcddb_conn, port);
                 }
-                else if (cdng_cfg.cddb_http)
+                else if (aud_get_bool ("CDDA", "cddbhttp"))
                 {
                     cddb_http_enable (pcddb_conn);
-                    cddb_set_server_name (pcddb_conn, cdng_cfg.cddb_server);
-                    cddb_set_server_port (pcddb_conn, cdng_cfg.cddb_port);
-                    cddb_set_http_path_query (pcddb_conn, cdng_cfg.cddb_path);
+                    cddb_set_server_name (pcddb_conn, server);
+                    cddb_set_server_port (pcddb_conn, port);
+                    cddb_set_http_path_query (pcddb_conn, path);
                 }
                 else
                 {
-                    cddb_set_server_name (pcddb_conn, cdng_cfg.cddb_server);
-                    cddb_set_server_port (pcddb_conn, cdng_cfg.cddb_port);
+                    cddb_set_server_name (pcddb_conn, server);
+                    cddb_set_server_port (pcddb_conn, port);
                 }
 
+                free (server);
+                free (path);
+
                 pcddb_disc = cddb_disc_new ();
 
                 lba = cdio_get_track_lba (pcdrom_drive->p_cdio,
@@ -827,13 +760,13 @@ static void scan_cd (void)
                 AUDDBG ("CDDB disc id = %x\n", discid);
 #endif
 
-                gint matches;
+                int matches;
                 if ((matches = cddb_query (pcddb_conn, pcddb_disc)) == -1)
                 {
                     if (cddb_errno (pcddb_conn) == CDDB_ERR_OK)
-                        cdaudio_error ("Failed to query the CDDB server");
+                        cdaudio_error (_("Failed to query the CDDB server"));
                     else
-                        cdaudio_error ("Failed to query the CDDB server: %s",
+                        cdaudio_error (_("Failed to query the CDDB server: %s"),
                                        cddb_error_str (cddb_errno
                                                        (pcddb_conn)));
 
@@ -857,7 +790,7 @@ static void scan_cd (void)
                         cddb_read (pcddb_conn, pcddb_disc);
                         if (cddb_errno (pcddb_conn) != CDDB_ERR_OK)
                         {
-                            cdaudio_error ("failed to read the cddb info: %s",
+                            cdaudio_error (_("Failed to read the cddb info: %s"),
                                            cddb_error_str (cddb_errno
                                                            (pcddb_conn)));
                             cddb_disc_destroy (pcddb_disc);
@@ -873,7 +806,7 @@ static void scan_cd (void)
                                                  cddb_disc_get_genre
                                                  (pcddb_disc));
 
-                            gint trackno;
+                            int trackno;
                             for (trackno = firsttrackno; trackno <= lasttrackno;
                                  trackno++)
                             {
@@ -909,7 +842,7 @@ static void scan_cd (void)
 }
 
 /* mutex must be locked */
-static void refresh_trackinfo (gboolean warning)
+static void refresh_trackinfo (bool_t warning)
 {
     trigger_monitor ();
 
@@ -956,15 +889,15 @@ static void refresh_trackinfo (gboolean warning)
 }
 
 /* thread safe (mutex may be locked) */
-static gint calculate_track_length (gint startlsn, gint endlsn)
+static int calculate_track_length (int startlsn, int endlsn)
 {
     return ((endlsn - startlsn + 1) * 1000) / 75;
 }
 
 /* thread safe (mutex may be locked) */
-static gint find_trackno_from_filename (const gchar * filename)
+static int find_trackno_from_filename (const char * filename)
 {
-    gint track;
+    int track;
 
     if (strncmp (filename, "cdda://?", 8) || sscanf (filename + 8, "%d",
                                                      &track) != 1)
