@@ -1,28 +1,29 @@
-/*  Audacious - Cross-platform multimedia player
- *  Copyright (C) 2005-2011  Audacious development team
+/*
+ * ui_gtk.c
+ * Copyright 2009 William Pitcock, Tomasz Moń, Michał Lipski, and John Lindgren
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; under version 3 of the License.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions, and the following disclaimer.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses>.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions, and the following disclaimer in the documentation
+ *    provided with the distribution.
  *
- *  The Audacious team does not consider modular code linking to
- *  Audacious or using our public API to be a derived work.
+ * This software is provided "as is" and without any warranty, express or
+ * implied. In no event shall the authors be liable for any damages arising from
+ * the use of this software.
  */
 
+#include <string.h>
+
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
 #include <audacious/debug.h>
 #include <audacious/drct.h>
-#include <audacious/gtk-compat.h>
 #include <audacious/i18n.h>
 #include <audacious/playlist.h>
 #include <audacious/plugin.h>
@@ -42,10 +43,12 @@
 #include "playlist_util.h"
 
 static const gchar * const gtkui_defaults[] = {
+ "infoarea_show_vis", "TRUE",
  "infoarea_visible", "TRUE",
  "menu_visible", "TRUE",
  "player_visible", "TRUE",
  "statusbar_visible", "TRUE",
+ "close_button_visible", "TRUE",
 
  "autoscroll", "TRUE",
  "playlist_columns", "title artist album queued length",
@@ -60,7 +63,6 @@ static const gchar * const gtkui_defaults[] = {
  "always_on_top", "FALSE",
  "save_window_position", "TRUE",
  "show_song_titles", "TRUE",
- "custom_playlist_colors", "FALSE",
  NULL};
 
 static PluginHandle * search_tool;
@@ -72,9 +74,11 @@ static gulong volume_change_handler_id;
 
 static GtkAccelGroup * accel;
 
-static GtkWidget * button_play, * button_pause, * button_stop, * slider,
- * label_time, * button_shuffle, * button_repeat, * search_button;
-static GtkWidget * window, * vbox_outer, * vbox, * menu_box, * menu, * infoarea, * statusbar;
+static GtkWidget * window, * vbox_outer, * menu_box, * menu, * toolbar, * vbox,
+ * infoarea, * statusbar;
+static GtkToolItem * menu_button, * search_button, * button_play, * button_stop,
+ * button_shuffle, * button_repeat;
+static GtkWidget * slider, * label_time;
 static GtkWidget * menu_main, * menu_rclick, * menu_tab;
 
 static GtkWidget * error_win = NULL;
@@ -93,6 +97,7 @@ static void ui_show_error (const gchar * text);
 AUD_IFACE_PLUGIN
 (
     .name = N_("GTK Interface"),
+    .domain = PACKAGE,
     .init = init,
     .cleanup = cleanup,
     .show = ui_show,
@@ -130,14 +135,22 @@ static gboolean window_delete()
     return TRUE;
 }
 
-static void button_open_pressed()
+static void button_open_pressed (void)
 {
-    audgui_run_filebrowser(TRUE);
+    audgui_run_filebrowser (TRUE);
 }
 
-static void button_add_pressed()
+static void button_add_pressed (void)
 {
-    audgui_run_filebrowser(FALSE);
+    audgui_run_filebrowser (FALSE);
+}
+
+static void button_play_pressed (void)
+{
+    if (aud_drct_get_playing () && ! aud_drct_get_paused ())
+        aud_drct_pause ();
+    else
+        aud_drct_play ();
 }
 
 static gboolean title_change_cb (void)
@@ -283,13 +296,13 @@ static void do_seek (gint time)
     }
 }
 
-static gboolean ui_slider_change_value_cb(GtkRange * range, GtkScrollType scroll)
+static gboolean ui_slider_change_value_cb (GtkRange * range,
+ GtkScrollType scroll, gdouble value)
 {
-    gint value = gtk_range_get_value (range);
     set_time_label (value, aud_drct_get_length ());
 
-    if (!slider_is_moving)
-        do_seek (gtk_range_get_value (range));
+    if (! slider_is_moving)
+        do_seek (value);
 
     return FALSE;
 }
@@ -355,22 +368,14 @@ static void set_slider_length (gint length)
 
 static void pause_cb (void)
 {
-    if (aud_drct_get_paused ())
-    {
-        gtk_widget_show (button_play);
-        gtk_widget_hide (button_pause);
-    }
-    else
-    {
-        gtk_widget_hide (button_play);
-        gtk_widget_show (button_pause);
-    }
+    gtk_tool_button_set_stock_id ((GtkToolButton *) button_play,
+     aud_drct_get_paused () ? GTK_STOCK_MEDIA_PLAY : GTK_STOCK_MEDIA_PAUSE);
 }
 
 static void ui_playback_begin (void)
 {
     pause_cb ();
-    gtk_widget_set_sensitive (button_stop, TRUE);
+    gtk_widget_set_sensitive ((GtkWidget *) button_stop, TRUE);
 
     if (delayed_title_change_source)
         g_source_remove (delayed_title_change_source);
@@ -411,67 +416,36 @@ static void ui_playback_stop (void)
     delayed_title_change_source = g_idle_add ((GSourceFunc) title_change_cb,
      NULL);
 
-    gtk_widget_show (button_play);
-    gtk_widget_hide (button_pause);
-    gtk_widget_set_sensitive (button_stop, FALSE);
+    gtk_tool_button_set_stock_id ((GtkToolButton *) button_play, GTK_STOCK_MEDIA_PLAY);
+    gtk_widget_set_sensitive ((GtkWidget *) button_stop, FALSE);
     gtk_widget_hide (slider);
     gtk_widget_hide (label_time);
 }
 
-static gboolean rclick_cb (GtkWidget * widget, GdkEventButton * event)
+static GtkToolItem * toolbar_button_add (GtkWidget * toolbar,
+ void (* callback) (void), const gchar * stock_id)
 {
-    if (event->button != 3)
-        return FALSE;
+    GtkToolItem * item = gtk_tool_button_new_from_stock (stock_id);
+    gtk_toolbar_insert ((GtkToolbar *) toolbar, item, -1);
 
-    if (event->type == GDK_BUTTON_PRESS && menu_main)
-        gtk_menu_popup ((GtkMenu *) menu_main, NULL, NULL, NULL, NULL,
-         event->button, event->time);
-
-    return TRUE;
+    g_signal_connect (item, "clicked", callback, NULL);
+    return item;
 }
 
-static GtkWidget *toolbar_button_add(GtkWidget * toolbar, void (*callback) (), const gchar * stock_id)
+static GtkToolItem * toggle_button_new (const gchar * icon, const gchar * alt,
+ void (* toggled) (GtkToggleToolButton * button))
 {
-    GtkWidget *icon;
-    GtkWidget *button = gtk_button_new();
-
-    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-    gtk_widget_set_can_focus(button, FALSE);
-
-    icon = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON);
-    gtk_container_add(GTK_CONTAINER(button), icon);
-
-    gtk_box_pack_start(GTK_BOX(toolbar), button, FALSE, FALSE, 0);
-    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(callback), NULL);
-    g_signal_connect (button, "button-press-event", (GCallback) rclick_cb, NULL);
-
-    return button;
-}
-
-static GtkWidget * toggle_button_new (const gchar * icon, const gchar * alt,
- void (* toggled) (GtkToggleButton * button, void * user), void * user)
-{
-    GtkWidget * button = gtk_toggle_button_new ();
-    gtk_widget_set_can_focus (button, FALSE);
-    gtk_button_set_relief ((GtkButton *) button, GTK_RELIEF_NONE);
+    GtkToolItem * item = gtk_toggle_tool_button_new ();
 
     if (! alt)
-        gtk_container_add ((GtkContainer *) button, gtk_image_new_from_stock
-         (icon, GTK_ICON_SIZE_BUTTON));
+        gtk_tool_button_set_stock_id ((GtkToolButton *) item, icon);
     else if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (), icon))
-        gtk_container_add ((GtkContainer *) button, gtk_image_new_from_icon_name
-         (icon, GTK_ICON_SIZE_BUTTON));
+        gtk_tool_button_set_icon_name ((GtkToolButton *) item, icon);
     else
-    {
-        GtkWidget * label = gtk_label_new (NULL);
-        gchar * markup = g_markup_printf_escaped ("<small>%s</small>", alt);
-        gtk_label_set_markup ((GtkLabel *) label, markup);
-        g_free (markup);
-        gtk_container_add ((GtkContainer *) button, label);
-    }
+        gtk_tool_button_set_label ((GtkToolButton *) item, alt);
 
-    g_signal_connect (button, "toggled", (GCallback) toggled, user);
-    return button;
+    g_signal_connect (item, "toggled", (GCallback) toggled, NULL);
+    return item;
 }
 
 static GtkWidget *markup_label_new(const gchar * str)
@@ -491,8 +465,18 @@ static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event, voi
     switch (event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK))
     {
       case 0:;
-        /* single-key shortcuts; must not interfere with text entry */
         GtkWidget * focused = gtk_window_get_focus ((GtkWindow *) window);
+
+        /* escape key returns focus to playlist */
+        if (event->keyval == GDK_KEY_Escape)
+        {
+            if (! focused || ! gtk_widget_is_ancestor (focused, (GtkWidget *) UI_PLAYLIST_NOTEBOOK))
+                gtk_widget_grab_focus (playlist_get_treeview (aud_playlist_get_active ()));
+
+            return FALSE;
+        }
+
+        /* single-key shortcuts; must not interfere with text entry */
         if (focused && GTK_IS_ENTRY (focused))
             return FALSE;
 
@@ -514,11 +498,11 @@ static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event, voi
         case 'b':
             aud_drct_pl_next ();
             return TRUE;
-        case GDK_Left:
+        case GDK_KEY_Left:
             if (aud_drct_get_playing ())
                 do_seek (aud_drct_get_time () - 5000);
             return TRUE;
-        case GDK_Right:
+        case GDK_KEY_Right:
             if (aud_drct_get_playing ())
                 do_seek (aud_drct_get_time () + 5000);
             return TRUE;
@@ -529,8 +513,8 @@ static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event, voi
       case GDK_CONTROL_MASK:
         switch (event->keyval)
         {
-          case GDK_ISO_Left_Tab:
-          case GDK_Tab:
+          case GDK_KEY_ISO_Left_Tab:
+          case GDK_KEY_Tab:
             aud_playlist_set_active ((aud_playlist_get_active () + 1) %
              aud_playlist_count ());
             break;
@@ -542,8 +526,8 @@ static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event, voi
       case (GDK_CONTROL_MASK | GDK_SHIFT_MASK):
         switch (event->keyval)
         {
-          case GDK_ISO_Left_Tab:
-          case GDK_Tab:
+          case GDK_KEY_ISO_Left_Tab:
+          case GDK_KEY_Tab:
             aud_playlist_set_active (aud_playlist_get_active () ?
              aud_playlist_get_active () - 1 : aud_playlist_count () - 1);
             break;
@@ -554,11 +538,11 @@ static gboolean window_keypress_cb (GtkWidget * widget, GdkEventKey * event, voi
       case GDK_MOD1_MASK:
         switch (event->keyval)
         {
-          case GDK_Left:
+          case GDK_KEY_Left:
             if (aud_drct_get_playing ())
                 do_seek (aud_drct_get_time () - 5000);
             break;
-          case GDK_Right:
+          case GDK_KEY_Right:
             if (aud_drct_get_playing ())
                 do_seek (aud_drct_get_time () + 5000);
             break;
@@ -579,13 +563,13 @@ static gboolean playlist_keypress_cb (GtkWidget * widget, GdkEventKey * event, v
     case 0:
         switch (event->keyval)
         {
-        case GDK_Escape:
+        case GDK_KEY_Escape:
             ui_playlist_notebook_position (GINT_TO_POINTER (aud_playlist_get_active ()), NULL);
             return TRUE;
-        case GDK_Delete:
+        case GDK_KEY_Delete:
             playlist_delete_selected ();
             return TRUE;
-        case GDK_Menu:
+        case GDK_KEY_Menu:
             popup_menu_rclick (0, event->time);
             return TRUE;
         }
@@ -616,28 +600,30 @@ static gboolean playlist_keypress_cb (GtkWidget * widget, GdkEventKey * event, v
 
 static void update_toggles (void * data, void * user)
 {
-    gtk_toggle_button_set_active ((GtkToggleButton *) button_repeat, aud_get_bool (NULL, "repeat"));
-    gtk_toggle_button_set_active ((GtkToggleButton *) button_shuffle, aud_get_bool (NULL, "shuffle"));
+    gtk_toggle_tool_button_set_active ((GtkToggleToolButton *) button_repeat,
+     aud_get_bool (NULL, "repeat"));
+    gtk_toggle_tool_button_set_active ((GtkToggleToolButton *) button_shuffle,
+     aud_get_bool (NULL, "shuffle"));
 }
 
-static void toggle_repeat (GtkToggleButton * button, void * unused)
+static void toggle_repeat (GtkToggleToolButton * button)
 {
-    aud_set_bool (NULL, "repeat", gtk_toggle_button_get_active (button));
+    aud_set_bool (NULL, "repeat", gtk_toggle_tool_button_get_active (button));
 }
 
-static void toggle_shuffle (GtkToggleButton * button, void * unused)
+static void toggle_shuffle (GtkToggleToolButton * button)
 {
-    aud_set_bool (NULL, "shuffle", gtk_toggle_button_get_active (button));
+    aud_set_bool (NULL, "shuffle", gtk_toggle_tool_button_get_active (button));
 }
 
-static void toggle_search_tool (GtkToggleButton * button, void * unused)
+static void toggle_search_tool (GtkToggleToolButton * button)
 {
-    aud_plugin_enable (search_tool, gtk_toggle_button_get_active (button));
+    aud_plugin_enable (search_tool, gtk_toggle_tool_button_get_active (button));
 }
 
 static gboolean search_tool_toggled (PluginHandle * plugin, void * unused)
 {
-    gtk_toggle_button_set_active ((GtkToggleButton *) search_button,
+    gtk_toggle_tool_button_set_active ((GtkToggleToolButton *) search_button,
      aud_plugin_get_enabled (plugin));
     return TRUE;
 }
@@ -687,11 +673,6 @@ static gboolean init (void)
 {
     search_tool = aud_plugin_lookup_basename ("search-tool");
 
-    GtkWidget *tophbox;         /* box to contain toolbar and shbox */
-    GtkWidget *buttonbox;       /* contains buttons like "open", "next" */
-    GtkWidget *shbox;           /* box for volume control + slider + time combo --nenolod */
-    GtkWidget *evbox;
-
     aud_config_set_defaults ("gtkui", gtkui_defaults);
 
     audgui_set_default_icon();
@@ -707,9 +688,7 @@ static gboolean init (void)
     window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_default_size ((GtkWindow *) window, w, h);
     gtk_window_set_keep_above ((GtkWindow *) window, aud_get_bool ("gtkui", "always_on_top"));
-#if GTK_CHECK_VERSION (3, 0, 0)
     gtk_window_set_has_resize_grip ((GtkWindow *) window, FALSE);
-#endif
 
     if (aud_get_bool ("gtkui", "save_window_position") && (x != -1 || y != -1))
         gtk_window_move ((GtkWindow *) window, x, y);
@@ -719,66 +698,70 @@ static gboolean init (void)
     accel = gtk_accel_group_new ();
     gtk_window_add_accel_group ((GtkWindow *) window, accel);
 
-    vbox_outer = gtk_vbox_new (FALSE, 0);
+    vbox_outer = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
     gtk_container_add ((GtkContainer *) window, vbox_outer);
 
-    menu_box = gtk_hbox_new (FALSE, 0);
+    menu_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_pack_start ((GtkBox *) vbox_outer, menu_box, FALSE, FALSE, 0);
-    show_menu (aud_get_bool ("gtkui", "menu_visible"));
 
-    tophbox = gtk_hbox_new(FALSE, 0);
-    gtk_box_pack_start ((GtkBox *) vbox_outer, tophbox, FALSE, FALSE, 0);
-
-    buttonbox = gtk_hbox_new(FALSE, 0);
+    toolbar = gtk_toolbar_new ();
+    gtk_toolbar_set_style ((GtkToolbar *) toolbar, GTK_TOOLBAR_ICONS);
+    GtkStyleContext * context = gtk_widget_get_style_context (toolbar);
+    gtk_style_context_add_class (context, GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
+    gtk_box_pack_start ((GtkBox *) vbox_outer, toolbar, FALSE, FALSE, 0);
 
+    /* search button */
     if (search_tool)
     {
-        search_button = toggle_button_new (GTK_STOCK_FIND, NULL, toggle_search_tool, NULL);
-        gtk_box_pack_start ((GtkBox *) tophbox, search_button, FALSE, FALSE, 0);
-        gtk_toggle_button_set_active ((GtkToggleButton *) search_button,
+        search_button = toggle_button_new (GTK_STOCK_FIND, NULL, toggle_search_tool);
+        gtk_toolbar_insert ((GtkToolbar *) toolbar, search_button, -1);
+        gtk_toggle_tool_button_set_active ((GtkToggleToolButton *) search_button,
          aud_plugin_get_enabled (search_tool));
         aud_plugin_add_watch (search_tool, search_tool_toggled, NULL);
     }
 
-    toolbar_button_add(buttonbox, button_open_pressed, GTK_STOCK_OPEN);
-    toolbar_button_add(buttonbox, button_add_pressed, GTK_STOCK_ADD);
-    button_play = toolbar_button_add (buttonbox, aud_drct_play, GTK_STOCK_MEDIA_PLAY);
-    button_pause = toolbar_button_add (buttonbox, aud_drct_pause, GTK_STOCK_MEDIA_PAUSE);
-    button_stop = toolbar_button_add (buttonbox, aud_drct_stop, GTK_STOCK_MEDIA_STOP);
-    toolbar_button_add (buttonbox, aud_drct_pl_prev, GTK_STOCK_MEDIA_PREVIOUS);
-    toolbar_button_add (buttonbox, aud_drct_pl_next, GTK_STOCK_MEDIA_NEXT);
+    /* playback buttons */
+    toolbar_button_add (toolbar, button_open_pressed, GTK_STOCK_OPEN);
+    toolbar_button_add (toolbar, button_add_pressed, GTK_STOCK_ADD);
+    button_play = toolbar_button_add (toolbar, button_play_pressed, GTK_STOCK_MEDIA_PLAY);
+    button_stop = toolbar_button_add (toolbar, aud_drct_stop, GTK_STOCK_MEDIA_STOP);
+    toolbar_button_add (toolbar, aud_drct_pl_prev, GTK_STOCK_MEDIA_PREVIOUS);
+    toolbar_button_add (toolbar, aud_drct_pl_next, GTK_STOCK_MEDIA_NEXT);
 
-    /* Workaround: Show the play and pause buttons and then hide them again in
-     * order to coax GTK into loading icons for them. -jlindgren */
-    gtk_widget_show_all (button_play);
-    gtk_widget_show_all (button_pause);
-    gtk_widget_hide (button_play);
-    gtk_widget_hide (button_pause);
+    /* time slider and label */
+    GtkToolItem * boxitem1 = gtk_tool_item_new ();
+    gtk_tool_item_set_expand (boxitem1, TRUE);
+    gtk_toolbar_insert ((GtkToolbar *) toolbar, boxitem1, -1);
 
-    gtk_widget_set_no_show_all (button_play, TRUE);
-    gtk_widget_set_no_show_all (button_pause, TRUE);
+    GtkWidget * box1 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+    gtk_container_add ((GtkContainer *) boxitem1, box1);
 
-    gtk_box_pack_start(GTK_BOX(tophbox), buttonbox, FALSE, FALSE, 0);
-
-    /* The slider and time display are packed in an event box so that they can
-     * be redrawn separately as they are updated. */
-    evbox = gtk_event_box_new ();
-    gtk_box_pack_start ((GtkBox *) tophbox, evbox, TRUE, TRUE, 0);
-
-    shbox = gtk_hbox_new(FALSE, 0);
-    gtk_container_add ((GtkContainer *) evbox, shbox);
-
-    slider = gtk_hscale_new(NULL);
+    slider = gtk_scale_new (GTK_ORIENTATION_HORIZONTAL, NULL);
+    gtk_range_set_increments ((GtkRange *) slider, 5000, 5000);
     gtk_scale_set_draw_value(GTK_SCALE(slider), FALSE);
     gtk_widget_set_size_request(slider, 120, -1);
+    gtk_widget_set_valign (slider, GTK_ALIGN_CENTER);
     gtk_widget_set_can_focus(slider, FALSE);
-    gtk_box_pack_start ((GtkBox *) shbox, slider, TRUE, TRUE, 6);
-    gtk_widget_set_no_show_all (slider, TRUE);
+    gtk_box_pack_start ((GtkBox *) box1, slider, TRUE, TRUE, 6);
 
     label_time = markup_label_new(NULL);
+    gtk_box_pack_end ((GtkBox *) box1, label_time, FALSE, FALSE, 6);
+
+    gtk_widget_set_no_show_all (slider, TRUE);
     gtk_widget_set_no_show_all (label_time, TRUE);
 
-    gtk_box_pack_end ((GtkBox *) shbox, label_time, FALSE, FALSE, 6);
+    /* repeat and shuffle buttons */
+    button_repeat = toggle_button_new ("media-playlist-repeat", "RP", toggle_repeat);
+    gtk_toolbar_insert ((GtkToolbar *) toolbar, button_repeat, -1);
+    button_shuffle = toggle_button_new ("media-playlist-shuffle", "SF", toggle_shuffle);
+    gtk_toolbar_insert ((GtkToolbar *) toolbar, button_shuffle, -1);
+
+    /* volume button */
+    GtkToolItem * boxitem2 = gtk_tool_item_new ();
+    gtk_toolbar_insert ((GtkToolbar *) toolbar, boxitem2, -1);
+
+    GtkWidget * box2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+    gtk_container_add ((GtkContainer *) boxitem2, box2);
 
     volume = gtk_volume_button_new();
     gtk_button_set_relief(GTK_BUTTON(volume), GTK_RELIEF_NONE);
@@ -789,32 +772,23 @@ static gboolean init (void)
     aud_drct_get_volume(&lvol, &rvol);
     gtk_scale_button_set_value(GTK_SCALE_BUTTON(volume), (lvol + rvol) / 2);
 
-    gtk_box_pack_end ((GtkBox *) tophbox, volume, FALSE, FALSE, 0);
-
-    button_shuffle = toggle_button_new ("media-playlist-shuffle", "SHUF",
-     toggle_shuffle, NULL);
-    gtk_box_pack_end ((GtkBox *) tophbox, button_shuffle, FALSE, FALSE, 0);
-    button_repeat = toggle_button_new ("media-playlist-repeat", "REP",
-     toggle_repeat, NULL);
-    gtk_box_pack_end ((GtkBox *) tophbox, button_repeat, FALSE, FALSE, 0);
+    gtk_box_pack_start ((GtkBox *) box2, volume, FALSE, FALSE, 0);
 
+    /* main UI layout */
     layout_load ();
 
     GtkWidget * layout = layout_new ();
     gtk_box_pack_start ((GtkBox *) vbox_outer, layout, TRUE, TRUE, 0);
 
-    vbox = gtk_vbox_new (FALSE, 6);
+    vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
     layout_add_center (vbox);
 
-    /* Create playlist notebook */
     ui_playlist_notebook_new ();
     gtk_box_pack_start ((GtkBox *) vbox, (GtkWidget *) UI_PLAYLIST_NOTEBOOK, TRUE, TRUE, 0);
 
-    if (aud_get_bool ("gtkui", "infoarea_visible"))
-    {
-        infoarea = ui_infoarea_new ();
-        gtk_box_pack_end (GTK_BOX(vbox), infoarea, FALSE, FALSE, 0);
-    }
+    /* optional UI elements */
+    show_menu (aud_get_bool ("gtkui", "menu_visible"));
+    show_infoarea (aud_get_bool ("gtkui", "infoarea_visible"));
 
     if (aud_get_bool ("gtkui", "statusbar_visible"))
     {
@@ -904,14 +878,43 @@ static void cleanup (void)
     layout_cleanup ();
 }
 
+static void menu_position_cb (GtkMenu * menu, int * x, int * y, int * push, void * button)
+{
+    int xorig, yorig, xwin, ywin;
+
+    gdk_window_get_origin (gtk_widget_get_window (window), & xorig, & yorig);
+    gtk_widget_translate_coordinates (button, window, 0, 0, & xwin, & ywin);
+
+    * x = xorig + xwin;
+    * y = yorig + ywin + gtk_widget_get_allocated_height (button);
+    * push = TRUE;
+}
+
+static void menu_button_cb (void)
+{
+    if (gtk_toggle_tool_button_get_active ((GtkToggleToolButton *) menu_button))
+        gtk_menu_popup ((GtkMenu *) menu_main, NULL, NULL, menu_position_cb,
+         menu_button, 0, gtk_get_current_event_time ());
+    else
+        gtk_widget_hide (menu_main);
+}
+
+static void menu_hide_cb (void)
+{
+    gtk_toggle_tool_button_set_active ((GtkToggleToolButton *) menu_button, FALSE);
+}
+
 void show_menu (gboolean show)
 {
     aud_set_bool ("gtkui", "menu_visible", show);
 
     if (show)
     {
+        /* remove menu button from toolbar and show menu bar */
         if (menu_main)
             gtk_widget_destroy (menu_main);
+        if (menu_button)
+            gtk_widget_destroy ((GtkWidget *) menu_button);
 
         if (! menu)
         {
@@ -919,11 +922,12 @@ void show_menu (gboolean show)
             g_signal_connect (menu, "destroy", (GCallback) gtk_widget_destroyed,
              & menu);
             gtk_widget_show (menu);
-            gtk_container_add ((GtkContainer *) menu_box, menu);
+            gtk_box_pack_start ((GtkBox *) menu_box, menu, TRUE, TRUE, 0);
         }
     }
     else
     {
+        /* hide menu bar and add menu item to toolbar */
         if (menu)
             gtk_widget_destroy (menu);
 
@@ -932,6 +936,17 @@ void show_menu (gboolean show)
             menu_main = make_menu_main (accel);
             g_signal_connect (menu_main, "destroy", (GCallback)
              gtk_widget_destroyed, & menu_main);
+            g_signal_connect (menu_main, "hide", (GCallback) menu_hide_cb, NULL);
+        }
+
+        if (! menu_button)
+        {
+            menu_button = gtk_toggle_tool_button_new_from_stock (AUD_STOCK_AUDACIOUS);
+            g_signal_connect (menu_button, "destroy", (GCallback)
+             gtk_widget_destroyed, & menu_button);
+            gtk_widget_show ((GtkWidget *) menu_button);
+            gtk_toolbar_insert ((GtkToolbar *) toolbar, menu_button, 0);
+            g_signal_connect (menu_button, "toggled", (GCallback) menu_button_cb, NULL);
         }
     }
 }
@@ -943,6 +958,8 @@ void show_infoarea (gboolean show)
     if (show && ! infoarea)
     {
         infoarea = ui_infoarea_new ();
+        g_signal_connect (infoarea, "destroy", (GCallback) gtk_widget_destroyed, & infoarea);
+        ui_infoarea_show_vis (aud_get_bool ("gtkui", "infoarea_show_vis"));
         gtk_box_pack_end ((GtkBox *) vbox, infoarea, FALSE, FALSE, 0);
         gtk_widget_show_all (infoarea);
     }
@@ -954,6 +971,12 @@ void show_infoarea (gboolean show)
     }
 }
 
+void show_infoarea_vis (gboolean show)
+{
+    aud_set_bool ("gtkui", "infoarea_show_vis", show);
+    ui_infoarea_show_vis (show);
+}
+
 void show_statusbar (gboolean show)
 {
     aud_set_bool ("gtkui", "statusbar_visible", show);
@@ -978,7 +1001,19 @@ void popup_menu_rclick (guint button, guint32 time)
      time);
 }
 
-void popup_menu_tab (guint button, guint32 time)
+void popup_menu_tab (guint button, guint32 time, int playlist)
 {
+    menu_tab_playlist_id = aud_playlist_get_unique_id (playlist);
     gtk_menu_popup ((GtkMenu *) menu_tab, NULL, NULL, NULL, NULL, button, time);
 }
+
+void activate_search_tool (void)
+{
+    if (! search_tool)
+        return;
+
+    if (! aud_plugin_get_enabled (search_tool))
+        aud_plugin_enable (search_tool, TRUE);
+
+    aud_plugin_send_message (search_tool, "grab focus", NULL, 0);
+}
